/*
  Copyright 2010-2011 Grégory Soutadé

  This file is part of KissCount.

  KissCount is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  KissCount is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with KissCount.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef IMPORTENGINE_H
#define IMPORTENGINE_H

#include <model/model.h>
#include <controller/KissCount.h>

class KissCount;

class ImportPattern {
public:
    wxString pattern;
    wxString account;
    wxString category;
} ;

#define NULL_IMPORT_PATTERN wxT("(nil)")

class ImportEngine {
public:
    ImportEngine();
    ~ImportEngine();

    // Get supported file extension. Example :
    // "OFX files (*.ofx)|*.ofx"
    virtual wxString GetFileExt();

    // Handle the file
    virtual bool HandleFile(const wxString& path, User* user, Database* db, KissCount* kiss)=0;

    // Parse the file and return accounts that doesn't match
    virtual std::vector<wxString> ParseFile();
    
    // Final Step
    virtual std::vector<Operation>* GetOperations(std::map<wxString, wxString>& accounts);

    void MatchPattern(wxString& key, Operation& op);
    int UpdatePattern(int pos);

    static wxString RemoveUnused(const wxString& s);
protected:
    Database* _db;
    User* _user;
    wxString _path;
    KissCount* _kiss;

    wxString _shortExt;
    wxString _longExt;

    std::map<wxString, wxString> _accounts;
    std::vector<wxString> _unresolvedAccounts;
    std::vector<Operation> _operations;
    std::map<wxString, wxString> _descriptions;

    void ApplyPattern(ImportPattern& pattern, Operation& op);
    wxString FindPattern(wxString& s1, wxString& s2);
};

#endif
